/* Copyright 1999-2006 Red Hat, Inc.
 * 
 * Originally part of gnome-lokkit.
 * 
 * This software is dual-licensed; see 'COPYING-lokkit' for details.
 *
 */

#include <dirent.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <libintl.h>
#include <locale.h>
#define N_(String) gettext(String)

#include <netinet/in.h>

#include <newt.h>
#include <popt.h>
#include <ctype.h>

char *selinux_mode="enforcing";
char *selinux_type="targeted";
int selinux_state = -1;

struct port {
	int start_port, end_port;
	char *proto;
};

struct devcb {
	char *name;
	newtComponent comp;
	char val;
};

#define CHAIN_NAME "RH-Firewall-1-INPUT"
#define MARK_OF_THE_MASQ 0x9

char *fwscript    = "/etc/sysconfig/iptables";
char *fw6script   = "/etc/sysconfig/ip6tables";
char *config      = "/etc/sysconfig/system-config-securitylevel";
char *seconfig    = "/etc/selinux/config";
char *oldseconfig = "/etc/sysconfig/selinux";

char *iptables_cfg = "/etc/sysconfig/iptables-config";
char *new_iptables_cfg = "/etc/sysconfig/new-iptables-config";

static int enabled = -1;
static int numports = 0, numdevs = 0, nummasqs = 0;
static char **devs = NULL;
static char **masqs = NULL;
static struct port *ports = NULL;

int get_selinux_state(char *mode) {
	if (mode == NULL)
		return -1;
	if (!strcasecmp(mode,"enforcing"))
		return 1;
	else if (!strcasecmp(mode,"permissive"))
		return 2;
	else if (!strcasecmp(mode,"disabled"))
		return 0;
	return -1;
}

char **getNetworkDevices(int *len) {
	int num = 0;
	DIR *dir;
	char **ret = malloc(10*sizeof(char *));
	struct dirent *dent;
	
	dir = opendir("/etc/sysconfig/network-scripts");
	if (dir) {
		while ((dent = readdir(dir))) {
			if (!strncmp(dent->d_name,"ifcfg-",6) && 
			    !strstr(dent->d_name,":") &&
			    !strstr(dent->d_name,"~") &&
			    strcmp(dent->d_name,"ifcfg-lo")) {
				if (num > 10)
					ret = realloc(ret, (num+1) * sizeof(char *));

				ret[num] = strdup(dent->d_name+6);
				num++;
			}
		}
		closedir(dir);
	}

	*len = num;

	if (ret) {
		qsort((void *)ret, num, sizeof(char *), 
		      (int (*)(const void *, const void *))strcmp);
	}

	if (!num)
		return NULL;
	else
		return ret;
}

void helpCallback(newtComponent co, void * tag) {
	char helpnormal[5000];        
	char helpcustom[8000];

	strcat(helpnormal, N_("\n"
			      "Firewall Configuration\n"
			      "\n"
			      "   A firewall sits between your computer and the network, \n"
			      "   and determines which resources on your computer remote\n"
			      "   users on the network are able to access. A\n"
			      "   properly configured firewall can greatly increase\n"
			      "   the out-of-the-box security of your system.\n"
			      "\n"));
	strcat(helpnormal, N_("   Choose the appropriate security level for your\n"
			      "   system.\n"
			      "\n"));
	strcat(helpnormal, N_("   Enabled -- By choosing enabled, your system will\n"
			      "   not accept connections from the outside that are\n"
			      "   not explicitly defined by you.\n"
			      "\n"));
	strcat(helpnormal, N_("   If you are connecting your system to the Internet,\n"
			      "   but do not plan to run a server, this is the\n"
			      "   safest choice. If additional services are needed,\n"
			      "   you can choose Customize to allow specific\n"
			      "   services through the firewall, or to implicitly\n"
			      "   trust traffic coming over certain interfaces.\n"
			      "\n"));
	strcat(helpnormal, N_("   Disabled -- Disabling the  firewall allows complete\n"
			      "   access and does no security checking. It is recommended\n"
			      "   that this only be selected if you are running on a\n"
			      "   trusted network (not the Internet), or if you plan\n"
			      "   to do more detailed firewall configuration later.\n"
			      "\n"));

	strcat(helpcustom, N_("\n"
			      "Firewall Customization\n"
			      "\n"
			      "   Choose which trusted devices and incoming services\n"
			      "   should be allowed for your network security\n"
			      "   settings.\n"
			      "\n"));
	strcat(helpcustom, N_("   Trusted Devices -- Checking these for any of your\n"
			      "   devices allows all traffic coming from that device\n"
			      "   to be allowed. For example, if you are running a\n"
			      "   local network, but are connecting to the Internet\n"
			      "   via a PPP dialup, you could check that eth0 is\n"
			      "   trusted to allow any traffic coming from your\n"
			      "   local network.\n"
			      "\n"));
	strcat(helpcustom, N_("   It is not recommended to enable this for devices\n"
			      "   that are connected to public networks, such as the\n"
			      "   Internet.\n"
			      "\n"));
	strcat(helpcustom, N_("   Allow Incoming -- Enabling these options allows the\n"
			      "   specified services to pass through the firewall.\n"
			      "   Note, during a workstation-class installation, the\n"
			      "   majority of these services are not present on the\n"
			      "   system.\n"
			      "\n"));
	strcat(helpcustom, N_("     * SSH -- Secure Shell (SSH) is a protocol for\n"
			      "       logging into and executing commands on remote\n"
			      "       machines. It provides secure encrypted\n"
			      "       communications. If you plan on accessing your\n"
			      "       machine remotely via SSH over a firewalled\n"
			      "       interface, enable this option. You need the\n"
			      "       openssh-server package installed for this\n"
			      "       option to be useful.\n"));
	strcat(helpcustom, N_("     * Telnet -- Telnet is a protocol for logging\n"
			      "       into remote machines. It is unencrypted, and\n"
			      "       provides little security from network snooping\n"
			      "       attacks. Enabling telnet is not recommended.\n"
			      "       You need the telnet-server package installed\n"
			      "       for this option to be useful.\n"));
	strcat(helpcustom, N_("     * WWW (HTTP) -- HTTP is the protocol used \n"
			      "       to serve Web pages. If you plan on\n"
			      "       making your Web server publicly available,\n"
			      "       enable this option. This option is not\n"
			      "       required for viewing pages locally or\n"
			      "       developing Web pages. You need the httpd\n"
			      "       package installed for this option to be\n"
			      "       useful.\n"));
	strcat(helpcustom, N_("     * Secure WWW (HTTPS) -- HTTPS is a modified HTTP\n"
			      "       used to serve Web pages when security is\n"
			      "       important.  Examples are sites that require logins\n"
			      "       like stores or web mail.  This option is not\n"
			      "       required for viewing pages locally or\n"
			      "       developing Web pages.  You need the httpd\n"
			      "       package installed for this option to be useful\n"));
	strcat(helpcustom, N_("     * Mail (SMTP) -- This allows incoming SMTP mail\n"
			      "       delivery. If you need to allow remote hosts to\n"
			      "       connect directly to your machine to deliver\n"
			      "       mail, enable this option. You do not need to\n"
			      "       enable this if you collect your mail from your\n"
			      "       ISP's server by POP3 or IMAP, or if you use a\n"
			      "       tool such as fetchmail. Note that an\n"
			      "       improperly configured SMTP server can allow\n"
			      "       remote machines to use your server to send\n"
			      "       spam.\n"));
	strcat(helpcustom, N_("     * FTP -- FTP is a protocol used for remote file\n"
			      "       transfer. If you plan on making your FTP\n"
			      "       server publicly available, enable this option.\n"
			      "       You need the vsftpd package installed for this\n"
			      "       option to be useful.\n"));
	strcat(helpcustom, N_("     * Samba Browsing -- This allows you to access\n"
			      "       access and participate in Windows file and printer\n"
			      "       sharing networks.  You need the Samba package\n"
			      "       installed for this option to be useful\n"));
	strcat(helpcustom, N_("     * NFSv4 -- This allowed you to use the NFS\n"
			      "       version 4 protocol via TCP to share files.  You will\n"
			      "       need to have the NFS tools installed and properly\n"
			      "       configure your NFS server for this option to be\n"
			      "       useful.\n"));
	strcat(helpcustom, N_("     * Other ports -- You can specify that other\n"
			      "       ports not listed here be allowed through the\n"
			      "       firewall. The format to use is\n"
			      "       'port:protocol'. For example, if you wanted to\n"
			      "       allow IMAP access through your firewall, you\n"
			      "       can specify 'imap:tcp'. You can also specify\n"
			      "       numeric ports explicitly; to allow UDP packets\n"
			      "       on port 1234 through, specify '1234:udp'. To\n"
			      "       specify multiple ports, separate them by\n"
			      "       commas.\n"));

	char *texttag = (char *)tag;

	if (!strcmp(texttag,"help")) {
		newtWinMessage(N_("Firewall Configuration"), N_("Ok"), helpnormal);
	} else {
		newtWinMessage(N_("Firewall Customization"), N_("Ok"), helpcustom);
	}
}

void add_module(const char *module)
{
	char *buf, *append_pt;
	FILE *in_f, *out_f;

	if ((in_f = fopen(iptables_cfg, "r")) == NULL)
		return;

	if ((out_f = fopen(new_iptables_cfg, "w")) == NULL) {
		fclose(in_f);
		return;
	}

	/* Make sure the buffer's long enough for the added module. */
	buf = malloc(sizeof(char)*(1024+strlen(module)+2));

	while ((buf = fgets(buf, 1024, in_f)) != NULL) {
		/* If it's an empty module list, we don't have to do anything
		 * special for the output.
		 */
		if (strncmp(buf, "IPTABLES_MODULES=\"\"", 19) == 0)
			fprintf(out_f, "IPTABLES_MODULES=\"%s\"\n", module);
		else if (strncmp(buf, "IPTABLES_MODULES=\"", 18) != 0)
			fprintf(out_f, buf);
		else {
			if (strstr(buf, module) != NULL)
				fprintf(out_f, buf);
			else {
				/* Find the closing double quote. */
				if ((append_pt = strrchr(buf, '"')) == NULL) {
					fclose(in_f);
					fclose(out_f);
					unlink(new_iptables_cfg);
					return;
				}

				/* Output the existing string with the ending
				 * double quote cut off, the new module added
				 * in, and then the closing double quote.
				 */
				fprintf (out_f, "%.*s %s\"\n", append_pt-buf,
					 buf, module);
			}
		}
	}

	fclose(in_f);
	fclose(out_f);
	rename(new_iptables_cfg, iptables_cfg);
}

void remove_module(const char *module)
{
	char *buf, *module_start;
	FILE *in_f, *out_f;

	if ((in_f = fopen(iptables_cfg, "r")) == NULL)
		return;

	if ((out_f = fopen(new_iptables_cfg, "w")) == NULL) {
		fclose(in_f);
		return;
	}

	buf = malloc(sizeof(char)*1024);

	while ((buf = fgets(buf, 1024, in_f)) != NULL) {
		if (strncmp(buf, "IPTABLES_MODULES=\"", 18) != 0)
			fprintf(out_f, buf);
		else {
			if ((module_start = strstr(buf, module)) == NULL)
				fprintf(out_f, buf);
			else
				fprintf (out_f, "%.*s%s", module_start-buf, buf,
					 module_start+strlen(module));
		}
	}

	fclose(in_f);
	fclose(out_f);
	rename(new_iptables_cfg, iptables_cfg);
}

void free_array(char ***array, int *num) {
	int i;

	if (NULL == array) return ;

	for (i = 0; (*array) && (*array)[i]; i++)
		free((*array)[i]);

	if ((*array))
		free((*array));

	(*array) = NULL;

	if (NULL != num)
		(*num) = 0;
}

void add_dev(char ***array, char *name, int *num) {
	int x, found = 0;
				
	if ((NULL == array) || (NULL == num)) return ;
        
	for (x = 0; (*array) && (*array)[x]; x++)
		if (!strcmp((*array)[x], name))
			found = 1;

	if (!found) {
		(*array) = realloc((*array), ((*num) + 2) *
				   sizeof(char *));
		(*array)[(*num)] = name;
		(*array)[(*num)+1] = 0;
		(*num)++;
	}
}

void free_ports() {
	if (ports)
		free(ports);

	ports = NULL;
	numports = 0;
}

static int is_numbers (char *name) {
	char *ch;

	for (ch = name; *ch != '\0'; ch++) {
		if (isalpha(*ch))
			return 0;
	}

	return 1;
}

void add_port(char *name) {
	char *tmp, *tmp2;
	char *protospec;
	struct servent *service;
	int x, found = 0, start_port = 0, end_port = 0;

	if ((tmp = strstr(name,":")) != NULL) {
		*tmp = '\0';
		tmp++;
		protospec = tmp;
	} else {
		protospec = "tcp";
	}

	/* If a dash is given in the port field, that means a range has been
	 * specified.  Split it out into starting and ending ports.  Otherwise,
	 * leave the end port at 0.
	 */
	if (is_numbers (name) && (tmp2 = strstr(name, "-")) != NULL) {
		*tmp2 = '\0';
		tmp2++;
		start_port = strtol(name, &tmp2-1, 10);
		end_port = strtol(tmp2, &tmp, 10);
	} else {
		start_port = strtol(name, &tmp, 10);
	}

	if (tmp && end_port == 0) {
		service = getservbyname(name, protospec);

		if (service) {
			start_port = ntohs(service->s_port);
			end_port = 0;
		}
	}

	if (start_port > 65535 || end_port > 65535) {
		fprintf(stderr, "Error: Port or port range %s is not valid\n",
			name);
		return;
	}

	if (start_port) {
		for (x = 0; ports && ports[x].start_port; x++)
			if (ports[x].start_port == start_port && !strcmp(ports[x].proto, protospec))
				found = 1;

		if (!found) {
			ports = realloc(ports, (numports+2) * sizeof(struct port));
			ports[numports].start_port = start_port;
			ports[numports].end_port = end_port;
			ports[numports].proto = protospec;
			ports[numports+1].start_port = 0;
			numports++;
		}
	}
}

char *add_portstr(char *orig, struct port *port) {
	char tmp[256];
	char *ret;
	struct servent *service;
	
	memset(tmp,'\0', 256);

        if (port->end_port == 0) {
		service = getservbyport(htons(port->start_port), port->proto);

		if (service)
			snprintf(tmp, 255, "%s:%s", service->s_name, port->proto);
		else
			snprintf(tmp, 255, "%d:%s", port->start_port, port->proto);
        } else
		snprintf(tmp, 255, "%d-%d:%s", port->start_port, port->end_port, port->proto);

	ret = calloc((orig ? strlen(orig) : 0) + strlen(tmp) + 2, sizeof(char));

	if (orig)
		sprintf(ret, "%s %s", orig, tmp);
	else
		sprintf(ret, "%s", tmp);

	return ret;
}

void runInterface() {
	newtComponent b_ok, b_custom, b_cancel, textbox, selinuxList;
	newtComponent enabled_rb, disabled_rb;
	newtComponent other, answer;
	struct devcb *dev_cbs, *masq_cbs;
	struct port *samba_ports[4] = { NULL, NULL, NULL, NULL };
	int samba_port_ndx = 0;
	char *portlist = NULL;
	char **devices = NULL;
	char *porttext = NULL;
	char roottext[80];
	int currentRow = 0, numdevices = 0, i, j;
	char ssh_state, telnet_state, smtp_state, http_state, https_state, ftp_state, samba_state;
	char nfs_state;
	int ssh = 0, telnet = 0, smtp = 0, http = 0, https = 0, ftp = 0, samba = 0, nfs = 0;
	newtGrid bb, toplevel, poplevel;
	newtGrid bigGrid, typeGrid, devGrid, portGrid, oGrid, selinuxGrid;

	bb = newtButtonBar (N_("OK"), &b_ok, N_("Customize"), &b_custom, N_("Cancel"), &b_cancel, NULL);

	toplevel = newtCreateGrid(1, 6);
	textbox = newtTextboxReflowed(-1, -1, N_("A firewall protects against unauthorized "
		 "network intrusions. Enabling a firewall blocks all "
		 "incoming connections. Disabling a firewall allows "
		 "all connections and is not recommended. "), 50, 5, 10, 0);

	bigGrid = newtCreateGrid(2,15);
	typeGrid = newtCreateGrid(3,2);
	selinuxGrid = newtCreateGrid(2, 1);

	newtGridSetField (typeGrid, 0, 0, NEWT_GRID_COMPONENT,
			  newtLabel(-1, -1, N_("Security Level:")),
			  0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);

	enabled_rb = newtRadiobutton(-1, -1, N_("Enabled"), enabled, NULL);
	newtGridSetField (typeGrid, 1, 0, NEWT_GRID_COMPONENT, enabled_rb,
			  0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	disabled_rb = newtRadiobutton(-1, -1, N_("Disabled"), !enabled, enabled_rb);
	newtGridSetField (typeGrid, 2, 0, NEWT_GRID_COMPONENT, disabled_rb,
			  0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);

	newtGridSetField (selinuxGrid, 0, 0, NEWT_GRID_COMPONENT,
	                  newtLabel(-1, -1, N_("SELinux:")),
			  0, 0, 1, 1, NEWT_ANCHOR_LEFT, 0);

	selinuxList = newtListbox(-1, -1, 3, 0);
	newtListboxAppendEntry(selinuxList, N_("Enforcing"), "enforcing");
	newtListboxAppendEntry(selinuxList, N_("Permissive"), "permissive");
	newtListboxAppendEntry(selinuxList, N_("Disabled"), "disabled");
	newtGridSetField (selinuxGrid, 1, 0, NEWT_GRID_COMPONENT, selinuxList,
	                  0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);

	if (!strcmp(selinux_mode, "enforcing"))
	   newtListboxSetCurrent(selinuxList, 0);
	else if (!strcmp(selinux_mode, "permissive"))
	   newtListboxSetCurrent(selinuxList, 1);
	else if (!strcmp(selinux_mode, "disabled"))
	   newtListboxSetCurrent(selinuxList, 2);

	currentRow = 0;
	devices = getNetworkDevices(&numdevices);

	if (devices) {
		newtComponent cb;
		int curcol, currow, rows, cols;

		if (numdevices > 4) {
			rows = (numdevices - 1) /4 + 1;
			cols = 4;
		} else {
			cols = numdevices;
			rows = 1;
		}

		devGrid = newtCreateGrid(cols, rows);

		newtGridSetField(bigGrid, 0, currentRow, NEWT_GRID_COMPONENT,
				 newtLabel(-1, -1, N_("Trusted Devices:")),
				 0, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
		curcol = 0;
		currow = 0;
		dev_cbs = malloc(numdevices * sizeof(struct devcb));

		for (i=0; i < numdevices; i++) {
			for (j = 0; devs && devs[j] ; j++)
				if (!strcmp(devs[j], devices[i])) {
					cb = newtCheckbox (-1, -1, devices[i], '*', NULL, &dev_cbs[i].val);
					break;
				}

			if (!devs || !devs[j])
				cb = newtCheckbox (-1, -1, devices[i], ' ', NULL, &dev_cbs[i].val);

			dev_cbs[i].name = devices[i];
			dev_cbs[i].comp = cb;

			newtGridSetField(devGrid, curcol, currow, NEWT_GRID_COMPONENT,
					 cb, 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
			curcol++;

			if (curcol >= cols) {
				currow++;
				curcol = 0;
			}
		}

		newtGridSetField (bigGrid, 1, currentRow, NEWT_GRID_SUBGRID, devGrid, 
				  1, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
		currentRow++;
/***End trusted devices****************************/

		devGrid = newtCreateGrid(cols, rows);

		newtGridSetField(bigGrid, 0, currentRow, NEWT_GRID_COMPONENT,
				 newtLabel(-1, -1, N_("MASQUERADE Devices:")),
				 0, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
		curcol = 0;
		currow = 0;
		masq_cbs = malloc(numdevices * sizeof(struct devcb));

		for (i=0; i < numdevices; i++) {
			for (j = 0; masqs && masqs[j] ; j++)
				if (!strcmp(masqs[j], devices[i])) {
					cb = newtCheckbox (-1, -1, devices[i], '*', NULL, &masq_cbs[i].val);
					break;
				}

			if (!masqs || !masqs[j])
				cb = newtCheckbox (-1, -1, devices[i], ' ', NULL, &masq_cbs[i].val);

			masq_cbs[i].name = devices[i];
			masq_cbs[i].comp = cb;

			newtGridSetField(devGrid, curcol, currow, NEWT_GRID_COMPONENT,
					 cb, 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
			curcol++;

			if (curcol >= cols) {
				currow++;
				curcol = 0;
			}
		}

		newtGridSetField (bigGrid, 1, currentRow, NEWT_GRID_SUBGRID, devGrid, 
				  1, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
		currentRow++;
/***End MASQUERADE devices*************************/
	}

	newtGridSetField (bigGrid, 0, currentRow, NEWT_GRID_COMPONENT,
			  newtLabel(-1, -1, N_("Allow incoming:")), 0, 0, 0, 0,
			  NEWT_ANCHOR_LEFT, 0);

	portGrid = newtCreateGrid(3,3);

	for (i = 0; ports && ports[i].start_port; i++) {
		if (!strcmp(ports[i].proto, "tcp")) {
			switch (ports[i].start_port) {
				case 22:
					ssh = 1;
					break;
				case 23:
					telnet = 1;
					break;
				case 80:
					http = 1;
					break;
				case 25:
					smtp = 1;
					break;
				case 21:
					ftp = 1;
					break;
				case 443:
					https = 1;
					break;
				case 139:
				case 445:
					samba_ports[samba_port_ndx] = &ports[i];
					samba_port_ndx++;
					break;
				case 2049:
					nfs = 1;
					break;
				default:
					porttext = add_portstr(porttext, &ports[i]);
					break;
			}
		} else {
			switch (ports[i].start_port) {
				case 137:
				case 138:
					samba_ports[samba_port_ndx] = &ports[i];
					samba_port_ndx++;
					break;
				default:
					porttext = add_portstr(porttext, &ports[i]);
					break;
			}
		}
	}

	/* If all the ports required for Samba browsing are listed, mark that box.
	 * Otherwise, just add the ports to the list.
	 */
	if (samba_port_ndx == 4)
		samba = 1;
	else
	{
		unsigned int i;

		for (i = 0; i < samba_port_ndx; i++)
			porttext = add_portstr(porttext, samba_ports[i]);
	}

	newtGridSetField(portGrid, 0, 0, NEWT_GRID_COMPONENT, 
			 newtCheckbox(-1, -1, N_("SSH"), ssh ? '*' : ' ', NULL, &ssh_state),
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(portGrid, 1, 0, NEWT_GRID_COMPONENT, 
			 newtCheckbox(-1, -1, N_("Telnet"), telnet ? '*': ' ', NULL, &telnet_state),
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(portGrid, 2, 0, NEWT_GRID_COMPONENT,
			 newtCheckbox(-1, -1, N_("FTP"), ftp ? '*' : ' ', NULL, &ftp_state),
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(portGrid, 0, 1, NEWT_GRID_COMPONENT, 
			 newtCheckbox(-1, -1, N_("WWW (HTTP)"), http ? '*' : ' ', NULL, &http_state),
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(portGrid, 1, 1, NEWT_GRID_COMPONENT,
			 newtCheckbox(-1, -1, N_("Samba"), samba ? '*' : ' ', NULL, &samba_state),
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(portGrid, 2, 1, NEWT_GRID_COMPONENT,
			 newtCheckbox(-1, -1, N_("Mail (SMTP)"), smtp ? '*' : ' ', NULL, &smtp_state),
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(portGrid, 0, 2, NEWT_GRID_COMPONENT, 
			 newtCheckbox(-1, -1, N_("Secure WWW (HTTPS)"), https ? '*' : ' ', NULL,
			 &https_state), 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(portGrid, 1, 2, NEWT_GRID_COMPONENT,
			 newtCheckbox(-1, -1, N_("NFS4"), nfs ? '*' : ' ', NULL, &nfs_state),
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);

	oGrid = newtCreateGrid(2,1);
	newtGridSetField (oGrid, 0, 0, NEWT_GRID_COMPONENT,
			  newtLabel(-1, -1, N_("Other ports")), 
			  0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	other = newtEntry (-1, -1, "", 25, (const char **) (&portlist),
			   NEWT_ENTRY_SCROLL);

	if (porttext)
		newtEntrySet(other, porttext, 0);

	newtGridSetField(oGrid, 1, 0, NEWT_GRID_COMPONENT, other,
			 0, 0, 1, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(bigGrid, 1, currentRow, NEWT_GRID_SUBGRID, portGrid,
			 1, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(bigGrid, 0, currentRow + 1, NEWT_GRID_COMPONENT,
			 newtLabel(-1, -1, ""), 0, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(bigGrid, 1, currentRow + 1, NEWT_GRID_SUBGRID,
			 oGrid, 1, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);

	newtGridSetField(toplevel, 0, 0, NEWT_GRID_COMPONENT, textbox,
			 0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
	newtGridSetField(toplevel, 0, 1, NEWT_GRID_SUBGRID, typeGrid,
			 0, 1, 0, 0, 0, 0);
	newtGridSetField(toplevel, 0, 2, NEWT_GRID_SUBGRID, selinuxGrid,
	                 0, 1, 0, 1, 0, 0);
	newtGridSetField(toplevel, 0, 3, NEWT_GRID_SUBGRID, bb,
			 0, 0, 0, 0, 0, NEWT_GRID_FLAG_GROWX);

	newtSetHelpCallback(helpCallback);

	newtInit();
        newtCls();
	newtPushHelpLine(N_(" <Tab>/<Alt-Tab> between elements   |   <Space> selects  |   <F12> next screen"));

	snprintf(roottext, 80, "system-config-securitylevel                     "
		 "      © 2003 Red Hat, Inc.");
	newtDrawRootText(0, 0, roottext);

	while (1) {
		newtComponent f;

		f = newtForm(NULL, "help", 0);
		newtGridWrappedWindow(toplevel, N_("Firewall Configuration"));

		newtGridAddComponentsToForm(toplevel, f, 1);

		answer = newtRunForm(f);

		if (answer == b_cancel) {
			newtPopWindow();
			newtFormDestroy(f);
			newtFinished();
			exit(0);
		}

		if (answer == b_custom) {
			newtComponent rb, popb_ok, pop_textbox, pop_f;
			newtGrid popbb;

			rb = newtRadioGetCurrent(enabled_rb);

			popbb = newtButtonBar (N_("OK"), &popb_ok, NULL);

			poplevel = newtCreateGrid(1, 3);

			pop_textbox = newtTextboxReflowed(-1, -1,
							  N_("You can customize your firewall in two ways. "
							    "First, you can select to allow all traffic from "
							    "certain network interfaces. Second, you can allow "
							    "certain protocols explicitly through the firewall. "
							    "Specify additional ports in the form 'service:protocol', "
							    "such as 'imap:tcp'. "), 
							  65, 5, 10, 0);

			newtGridSetField (poplevel, 0, 0, NEWT_GRID_COMPONENT,
					  pop_textbox, 0, 0, 0, 1, NEWT_ANCHOR_LEFT, 0);
			newtGridSetField (poplevel, 0, 1, NEWT_GRID_SUBGRID,
					  bigGrid, 0, 0, 0, 0, NEWT_ANCHOR_LEFT, 0);
			newtGridSetField (poplevel, 0, 2, NEWT_GRID_SUBGRID,
					  popbb, 0, 0, 0, 0, NEWT_ANCHOR_LEFT, NEWT_GRID_FLAG_GROWX);

			pop_f = newtForm(NULL, "help_custom", 0);

			newtGridWrappedWindow(poplevel,N_("Firewall Configuration - Customize"));
			newtGridAddComponentsToForm(poplevel, pop_f, 1);

			newtRunForm(pop_f);
			newtPopWindow();
		} else {
		    /* must have pressed F12 */
		    break;
		}

		if (answer == b_ok)
			break;

		newtPopWindow();
	}

        newtPopWindow();
	newtFinished();

	free_array(&devs, &numdevs);
	free_array(&masqs, &nummasqs);
	free_ports();

	if (devices) {
		for (i = 0; i < numdevices ; i++) {
			if (dev_cbs[i].val == '*') {
				add_dev(&devs, dev_cbs[i].name, &numdevs);
			}
		}

		for (i = 0; i < numdevices ; i++) {
			if (masq_cbs[i].val == '*') {
				if (dev_cbs[i].val == '*') {
					add_dev(&masqs, masq_cbs[i].name, &nummasqs);
				} else {
					fprintf (stderr, "Will not add %s to MASQUERADE list because it is not trusted.\n", masq_cbs[i].name) ;
				}
			}
		}
	}

	if (portlist) {
		char *tmp = portlist;
		while (*portlist) {
			while (*tmp && *tmp != ' ' && *tmp != ',')
				tmp++;

			if (*tmp) {
				*tmp = '\0';
				tmp++;
			}

			while (*tmp && (*tmp == ' ' || *tmp == ','))
				tmp++;

			add_port(portlist);
			portlist = tmp;
		}
	}

	if (ssh_state == '*')
		add_port(strdup("ssh:tcp"));

	if (telnet_state == '*')
		add_port(strdup("telnet:tcp"));

	if (smtp_state == '*')
		add_port(strdup("smtp:tcp"));

	if (http_state == '*')
		add_port(strdup("http:tcp"));

	if (ftp_state == '*') {
		add_port(strdup("ftp:tcp"));
		add_module("ip_conntrack_ftp");
	}
	else
		remove_module("ip_conntrack_ftp");

	if (https_state == '*')
		add_port(strdup("https:tcp"));

	if (samba_state == '*') {
		add_port(strdup("137:udp"));
		add_port(strdup("138:udp"));
		add_port(strdup("139:tcp"));
		add_port(strdup("445:tcp"));
	}

	if (nfs_state == '*') {
		add_port(strdup("2049:tcp"));
	}
	
	answer = newtRadioGetCurrent(enabled_rb);

	if (answer == enabled_rb) {
		enabled = 1;
	} else {
		enabled = 0;
	}

	selinux_mode = strdup(newtListboxGetCurrent(selinuxList));
	selinux_state = get_selinux_state(selinux_mode);
}

int write_firewall(char **devs, char **masqs, struct port *ports,
		   int activate)
{
	int i;
	FILE *fw = NULL;
	char buffer[128];

	fw = fopen(fwscript, "w");

	if (fw==NULL)
		return -1;

	fprintf(fw, "# Firewall configuration written by system-config-securitylevel\n");
	fprintf(fw, "# Manual customization of this file is not recommended.\n");
	fprintf(fw, "*filter\n");
	fprintf(fw, ":INPUT ACCEPT [0:0]\n");
	fprintf(fw, ":FORWARD ACCEPT [0:0]\n");
	fprintf(fw, ":OUTPUT ACCEPT [0:0]\n");
	fprintf(fw, ":%s - [0:0]\n", CHAIN_NAME);
	fprintf(fw, "-A INPUT -j %s\n", CHAIN_NAME);
	fprintf(fw, "-A FORWARD -j %s\n", CHAIN_NAME);

	fprintf(fw, "-A %s -i lo -j ACCEPT\n", CHAIN_NAME);

	for (i=0; devs && devs[i]; i++)
		fprintf(fw, "-A %s -i %s -j ACCEPT\n", CHAIN_NAME, devs[i]);

	/* Allow ICMP. */
	fprintf(fw, "-A %s -p icmp --icmp-type any -j ACCEPT\n", CHAIN_NAME);

	/* Allow IPSec traffic. This matches Lokkit 0.50 behavior and seems reasonable. */
	fprintf(fw, "-A %s -p 50 -j ACCEPT\n", CHAIN_NAME);
	fprintf(fw, "-A %s -p 51 -j ACCEPT\n", CHAIN_NAME);

	/* Allow mDNS. */
	fprintf(fw, "-A %s -p udp --dport 5353 -d 224.0.0.251 -j ACCEPT\n", CHAIN_NAME);

	/* Allow CUPS browsing */
	fprintf(fw, "-A %s -p udp -m udp --dport 631 -j ACCEPT\n", CHAIN_NAME);
	fprintf(fw, "-A %s -p tcp -m tcp --dport 631 -j ACCEPT\n", CHAIN_NAME);

	/* Note: The RELATED is extremely important as it matches ICMP error messages. */
	fprintf(fw, "-A %s -m state --state ESTABLISHED,RELATED -j ACCEPT\n", CHAIN_NAME);

	for (i=0; ports && ports[i].start_port; i++) {
                if (ports[i].end_port == 0)
		        fprintf(fw, "-A %s -m state --state NEW -m %s -p %s --dport %d -j ACCEPT\n",
			        CHAIN_NAME, ports[i].proto, ports[i].proto, ports[i].start_port);
                else
		        fprintf(fw, "-A %s -m state --state NEW -m %s -p %s --dport %d:%d -j ACCEPT\n",
			        CHAIN_NAME, ports[i].proto, ports[i].proto, ports[i].start_port,
                                ports[i].end_port);
        }

	fprintf(fw, "-A %s -j REJECT --reject-with icmp-host-prohibited\n", CHAIN_NAME);

	/* Mark packets coming from MASQUERADE devices for MASQERADEing */
	if (masqs) {
		fprintf (fw, "COMMIT\n");
		fprintf (fw, "*mangle\n");
		fprintf(fw, ":PREROUTING ACCEPT [0:0]\n");
		fprintf(fw, ":INPUT ACCEPT [0:0]\n");
		fprintf(fw, ":FORWARD ACCEPT [0:0]\n");
		fprintf(fw, ":OUTPUT ACCEPT [0:0]\n");
		fprintf(fw, ":POSTROUTING ACCEPT [0:0]\n");

		for (i=0; masqs && masqs[i]; i++)
			fprintf (fw, "-A PREROUTING -i %s -j MARK --set-mark 0x%X\n", masqs[i], MARK_OF_THE_MASQ);

		fprintf (fw, "COMMIT\n");
		fprintf (fw, "*nat\n");
		fprintf(fw, ":PREROUTING ACCEPT [0:0]\n");
		fprintf(fw, ":OUTPUT ACCEPT [0:0]\n");
		fprintf(fw, ":POSTROUTING ACCEPT [0:0]\n");
		fprintf (fw, "-A POSTROUTING -m mark --mark 0x%X -j MASQUERADE\n", MARK_OF_THE_MASQ) ;
	}

	fprintf(fw, "COMMIT\n");

	fclose(fw);
	chmod(fwscript,0600);

	/* Make sure certain iptables modules are always loaded.  These
	 * shouldn't cause any problems if loaded.
	 */
	add_module("ip_conntrack_netbios_ns");

	if (activate) {
		/* The 2.4 kernel won't load this on demand, apparently. */
		sprintf(buffer,"/sbin/modprobe iptables >/dev/null 2>&1");
		system(buffer);

		sprintf(buffer, "/sbin/service iptables start >/dev/null 2>&1");
		system(buffer);
	}
	return 0;
}

/* 
Implement rules for ip6tables.
Since there is no need to re-implement rules below layer 3, some steps
performed by write_firewall(), such as allowing the loopback interface, 
may be omitted
*/
int write_firewall6(char **devs, char **masqs, struct port *ports,
		   int activate)
{	
	int i;
	FILE *fw = NULL;
	char buffer[128];

	fw = fopen(fw6script, "w");

	if (fw==NULL)
		return -1;

	fprintf(fw, "# Firewall configuration written by system-config-securitylevel\n");
	fprintf(fw, "# Manual customization of this file is not recommended.\n");
	fprintf(fw, "*filter\n");
	fprintf(fw, ":INPUT ACCEPT [0:0]\n");
	fprintf(fw, ":FORWARD ACCEPT [0:0]\n");
	fprintf(fw, ":OUTPUT ACCEPT [0:0]\n");
	fprintf(fw, ":%s - [0:0]\n", CHAIN_NAME);
	fprintf(fw, "-A INPUT -j %s\n", CHAIN_NAME);
	fprintf(fw, "-A FORWARD -j %s\n", CHAIN_NAME);

	fprintf(fw, "-A %s -i lo -j ACCEPT\n", CHAIN_NAME);

	for (i=0; devs && devs[i]; i++)
		fprintf(fw, "-A %s -i %s -j ACCEPT\n", CHAIN_NAME, devs[i]);

	/* Allow ICMP. */
	fprintf(fw, "-A %s -p icmpv6 -j ACCEPT\n", CHAIN_NAME);

	/* Allow IPSec traffic. This matches Lokkit 0.50 behavior and seems reasonable. */
	fprintf(fw, "-A %s -p 50 -j ACCEPT\n", CHAIN_NAME);
	fprintf(fw, "-A %s -p 51 -j ACCEPT\n", CHAIN_NAME);

	/* Allow mDNS. */
	fprintf(fw, "-A %s -p udp --dport 5353 -d ff02::fb -j ACCEPT\n", CHAIN_NAME);

	/* Allow CUPS browsing */
	fprintf(fw, "-A %s -p udp -m udp --dport 631 -j ACCEPT\n", CHAIN_NAME);
	fprintf(fw, "-A %s -p tcp -m tcp --dport 631 -j ACCEPT\n", CHAIN_NAME);

	/* Allow outgoing connections */
	fprintf(fw, "-A %s -p udp -m udp --dport 32768:61000 -j ACCEPT\n", CHAIN_NAME);
	fprintf(fw, "-A %s -p tcp -m tcp --dport 32768:61000 ! --syn -j ACCEPT\n", CHAIN_NAME);

	/* Note: The RELATED is extremely important as it matches ICMP error messages. */
	/* fprintf(fw, "-A %s -m state --state ESTABLISHED,RELATED -j ACCEPT\n", CHAIN_NAME); */

	for (i=0; ports && ports[i].start_port; i++) {
                if (ports[i].end_port == 0) {
			fprintf(fw, "-A %s -m %s -p %s --dport %d -j ACCEPT\n",
			        CHAIN_NAME, ports[i].proto, ports[i].proto,
				ports[i].start_port);
		}
/*
		        fprintf(fw, "-A %s -m state --state NEW -m %s -p %s --dport %d -j ACCEPT\n",
			        CHAIN_NAME, ports[i].proto, ports[i].proto, ports[i].start_port);
*/
                else {
			fprintf(fw, "-A %s -m %s -p %s --dport %d:%d -j ACCEPT\n",
			        CHAIN_NAME, ports[i].proto, ports[i].proto,
				ports[i].start_port, ports[i].end_port);
		}
/*
		        fprintf(fw, "-A %s -m state --state NEW -m %s -p %s --dport %d:%d -j ACCEPT\n",
			        CHAIN_NAME, ports[i].proto, ports[i].proto, ports[i].start_port,
                                ports[i].end_port);
*/
        }


	fprintf(fw, "-A %s -j REJECT --reject-with icmp6-adm-prohibited\n", CHAIN_NAME);

	/* The nat table has not yet been implemented for ipv6. 
	   The following should be uncommented when it has
	*/
#if 0
	if (masqs) {
		fprintf (fw, "COMMIT\n");
		fprintf (fw, "*mangle\n");
		fprintf(fw, ":PREROUTING ACCEPT [0:0]\n");
		fprintf(fw, ":INPUT ACCEPT [0:0]\n");
		fprintf(fw, ":FORWARD ACCEPT [0:0]\n");
		fprintf(fw, ":OUTPUT ACCEPT [0:0]\n");
		fprintf(fw, ":POSTROUTING ACCEPT [0:0]\n");

		for (i=0; masqs && masqs[i]; i++)
			fprintf (fw, "-A PREROUTING -i %s -j MARK --set-mark 0x%X\n", masqs[i], MARK_OF_THE_MASQ);

		fprintf (fw, "COMMIT\n");
		fprintf (fw, "*nat\n");
		fprintf(fw, ":PREROUTING ACCEPT [0:0]\n");
		fprintf(fw, ":OUTPUT ACCEPT [0:0]\n");
		fprintf(fw, ":POSTROUTING ACCEPT [0:0]\n");
		fprintf (fw, "-A POSTROUTING -m mark --mark 0x%X -j MASQUERADE\n", MARK_OF_THE_MASQ) ;
	}
#endif
	fprintf(fw, "COMMIT\n");

	fclose(fw);
	chmod(fw6script,0600);

	if (activate) {
		/* The 2.4 kernel won't load this on demand, apparently. */
		sprintf(buffer,"/sbin/modprobe ip6_tables >/dev/null 2>&1");
		system(buffer);

		sprintf(buffer, "/sbin/service ip6tables start >/dev/null 2>&1");
		system(buffer);
	}	
	return 0;
}


int write_config(char **devs, char **masqs, struct port *ports, int enabled)
{
	FILE *cfg = NULL;
	int i;

	cfg = fopen(config, "w");

	if (cfg == NULL)
		return -1;

	fprintf(cfg, "# Configuration file for system-config-securitylevel\n");
	fprintf(cfg, "\n");

	if (enabled == 0)
		fprintf(cfg, "--disabled\n");
	else if (enabled == 1)
		fprintf(cfg, "--enabled\n");

	/* Write out the config even if the firewall is disabled so we don't
	 * forget everyone's settings between runs (#183442).
	 */
	for (i=0; devs && devs[i]; i++)
		fprintf(cfg, "--trust=%s\n",devs[i]);
	for (i=0; masqs && masqs[i]; i++)
		fprintf(cfg, "--masq=%s\n",masqs[i]);
	for (i=0; ports && ports[i].start_port; i++) {
                if (ports[i].end_port == 0)
        		fprintf(cfg, "--port=%d:%s\n", ports[i].start_port, ports[i].proto);
                else
        		fprintf(cfg, "--port=%d-%d:%s\n", ports[i].start_port,
                                ports[i].end_port, ports[i].proto);
        }

	fclose(cfg);
	return 0;
}

int read_config(char ***fileargv)
{
	int numargs = 0; 
	char **args  = NULL;
	FILE *cfg;
	char *buf;

	cfg = fopen(config, "r");

	if (cfg) {
		buf = malloc(2048);
		memset(buf,'\0',2048);

		while ((buf = fgets(buf, 2048, cfg)) != NULL) {
			if (buf[0] == '#')
				continue;

			if (buf[strlen(buf)-1] == '\n')
				buf[strlen(buf)-1] = '\0';

			args = realloc(args,(numargs+2)*sizeof(char *));
			args[numargs] = strdup(buf);
			args[numargs+1] = NULL;
			numargs++;
		}
		fclose(cfg);
	}

	cfg = fopen(seconfig,"r");

	if (!cfg) {
	  cfg = fopen(oldseconfig,"r");
	}

	if (cfg) {
		int i;

		buf = malloc(2048);
		memset(buf,'\0',2048);

		while ((buf = fgets(buf, 2048, cfg)) != NULL) {
			if (strncmp(buf,"SELINUX=",8)==0) {
				args = realloc(args,(numargs+2)*sizeof(char *));
				args[numargs] = strdup(buf+8);
				selinux_mode=strdup(buf+8);
				i=strlen(selinux_mode)-1;

				while (i>=0 && isspace(selinux_mode[i]))
					selinux_mode[i--]=0;

				args[numargs+1] = NULL;
				numargs++;
			}

			if (strncmp(buf,"SELINUXTYPE=",12)==0) {
				args = realloc(args,(numargs+2)*sizeof(char *));
				args[numargs] = strdup(buf+12);
				selinux_type=strdup(buf+12);
				i=strlen(selinux_type)-1;

				while(i>=0 && isspace(selinux_type[i]))
				  selinux_type[i--]=0;

				args[numargs+1] = NULL;
				numargs++;
			}
		}
	}

	*fileargv = args;
	return 0;
}

int write_selinux(int state, int activate, char *type) {
	FILE *secfg;

	secfg = fopen(seconfig,"w");

	if (!secfg)
		return -1;

	fprintf(secfg, "# This file controls the state of SELinux on the system.\n");
	fprintf(secfg, "# SELINUX= can take one of these three values:\n");
	fprintf(secfg, "#\tenforcing - SELinux security policy is enforced.\n");
	fprintf(secfg, "#\tpermissive - SELinux prints warnings instead of enforcing.\n");
	fprintf(secfg, "#\tdisabled - SELinux is fully disabled.\n");

	if (state == 1) {
		fprintf(secfg,"SELINUX=enforcing\n");
		if (activate)
			system("/usr/sbin/setenforce 1");
	}
	if (state == 2) {
		fprintf(secfg,"SELINUX=permissive\n");
		if (activate)
			system("/usr/sbin/setenforce 0");
	}
	if (state == 0) {
		fprintf(secfg,"SELINUX=disabled\n");
		if (activate)
			system("/usr/sbin/setenforce 0");
	}

	if (type != "") {
		fprintf(secfg, "# SELINUXTYPE= type of policy in use. Possible values are:\n");
		fprintf(secfg, "#\ttargeted - Only targeted network daemons are protected.\n");
		fprintf(secfg, "#\tstrict - Full SELinux protection.\n");
		fprintf(secfg,"SELINUXTYPE=%s\n",type);
	}

	fclose(secfg);
	return 0;
}

int main(int argc, char **argv) {
	int tmp = 0;
	int ks = 0, nostart = 0, force = 0;
	char *portspec, *devspec, *masqspec, *modulespec;
	poptContext context;
	int rc;
	char **fileargv;

	struct poptOption options[] = {
		POPT_AUTOHELP
		{ "quiet", 'q', POPT_ARG_NONE, &ks, 0,
		  N_("Run noninteractively; process only command-line arguments"),
		  NULL
		},
	        { "nostart", 'n', POPT_ARG_NONE, &nostart, 0,
		  N_("Configure firewall but do not activate it"),
		  NULL
		},
		{ "enabled", 0, POPT_ARG_VAL, &enabled, 1,
		  N_("Enable firewall (default)"),
		  NULL,
		},
		{ "disabled", 0, POPT_ARG_VAL, &enabled, 0,
		  N_("Disable firewall"),
		  NULL
		},
		{ "addmodule", 0, POPT_ARG_STRING, &modulespec, 6,
		  N_("Enable an iptables module"),
		  N_("module to add")
		},
		{ "removemodule", 0, POPT_ARG_STRING, &modulespec, 7,
		  N_("Disable an iptables module"),
		  N_("module to remove")
		},
		{ "port",  'p', POPT_ARG_STRING, &portspec, 1,
		  N_("Allow specific ports through the firewall"),
		  N_("port:protocol (e.g, ssh:tcp)")
		},
		{ "trust", 't', POPT_ARG_STRING, &devspec, 2,
		  N_("Allow all traffic on the specified device"),
		  N_("device to trust")
		},
		{ "masq", 't', POPT_ARG_STRING, &masqspec, 5,
		  N_("Masquerades traffic from the specified device"),
		  N_("device to masquerade")
		},
                { "selinux", 0, POPT_ARG_STRING, &selinux_mode, 3,
 		  N_("Configure SELinux"),
		  N_("'enforcing', 'permissive', or 'disabled'"),
		},
		{ "selinuxtype", 0, POPT_ARG_STRING, &selinux_type, 4,
		  N_("Configure SELinux Policy"),
		  N_("Usually 'targeted' or 'strict'")
		},
		{ NULL, 'f', POPT_ARG_NONE, &force, 0,
		  N_("ignore settings stored in /etc/sysconfig/system-config-securitylevel"),
		  NULL,
		},
		{ "high", 0, POPT_ARG_NONE, &enabled, 0,
		  N_("Backwards compatibilty, aliased to --enabled"),
		  NULL,
		},
		{ "medium", 0, POPT_ARG_NONE, &enabled, 0,
		  N_("Backwards compatibilty, aliased to --enabled"),
		  NULL,
		},
		{ "dhcp", 0, POPT_ARG_NONE, &tmp, 0,
		  N_("Backwards compatiblity, ignored"),
		  NULL,
		},
		{ 0, 0, 0, 0, 0, 0 }
	};
	
	setlocale(LC_ALL, "");
	bindtextdomain("system-config-securitylevel", "/usr/share/locale");
	textdomain("system-config-securitylevel");

	if (getuid()) {
		fprintf(stderr,
			N_("\nERROR - You must be root to run lokkit.\n"));
		exit(1);
	}

	context = poptGetContext("lokkit", argc, (const char **) argv,
				 options, 0);

	/* Check for force flag before doing the real arg processing. */
	while (poptGetNextOpt(context) >= 0)
		;
	poptResetContext(context);

	if (!force) {
                read_config(&fileargv);
		if (fileargv != NULL) {
			poptStuffArgs(context, (const char **)fileargv);
		}
	}

	while ((rc = poptGetNextOpt(context)) >= 0) {
		if (rc == 1) {
			add_port(portspec);
		} else if (rc == 2) {
			add_dev(&devs, devspec, &numdevs);
		} else if (rc == 5) {
			add_dev(&masqs, masqspec, &nummasqs);
		} else if (rc == 3) {
			selinux_state=get_selinux_state(selinux_mode);
			if (selinux_state<0) {
				fprintf(stderr, "%s: %s\n",
					poptBadOption(context, POPT_BADOPTION_NOALIAS),
					poptStrerror(POPT_ERROR_BADOPT));
				exit(1);
			}
		}
		else if (rc == 4) {
			selinux_state=get_selinux_state(selinux_mode);
		}
		else if (rc == 6) {
			add_module(modulespec);
		}
		else if (rc == 7) {
			remove_module(modulespec);
		}
	}

	if (rc < -1) {
		fprintf(stderr, "%s: %s\n",
 			poptBadOption(context, POPT_BADOPTION_NOALIAS),
			poptStrerror(rc));
		exit(1);
	}

	if (ks) {
		if (enabled == 1) {
			write_firewall(devs, masqs, ports, !nostart);
			write_firewall6(devs, masqs, ports, !nostart);
		}
		else if (enabled == 0) {
			unlink(fwscript);
			unlink(fw6script);
		}

		if (selinux_state >= 0)
			write_selinux(selinux_state, !nostart, selinux_type);
	} else {
		if (enabled == -1)
			enabled = 1;

		runInterface();

		if (enabled) {
			write_firewall(devs, masqs, ports, !nostart);
			write_firewall6(devs, masqs, ports, !nostart);
		}
		if (selinux_state >= 0)
			write_selinux(selinux_state, !nostart, selinux_type);
	}

	if (!enabled && !nostart) {
		char path[256];
		
		snprintf(path, 255,"/sbin/iptables -D INPUT -j %s 2>/dev/null", CHAIN_NAME);
		system(path);
		snprintf(path, 255,"/sbin/iptables -D FORWARD -j %s 2>/dev/null", CHAIN_NAME);
		system(path);
		snprintf(path, 255,"/sbin/iptables -F %s 2>/dev/null", CHAIN_NAME);
		system(path);
		snprintf(path, 255,"/sbin/iptables -X %s 2>/dev/null", CHAIN_NAME);
		system(path);
		unlink(fwscript);

		snprintf(path, 255,"/sbin/ip6tables -D INPUT -j %s 2>/dev/null", CHAIN_NAME);
		system(path);
		snprintf(path, 255,"/sbin/ip6tables -D FORWARD -j %s 2>/dev/null", CHAIN_NAME);
		system(path);
		snprintf(path, 255,"/sbin/ip6tables -F %s 2>/dev/null", CHAIN_NAME);
		system(path);
		snprintf(path, 255,"/sbin/ip6tables -X %s 2>/dev/null", CHAIN_NAME);
		system(path);
		unlink(fw6script);
	}

	write_config(devs, masqs, ports, enabled);
	exit(0);
}

/* vim:set noexpandtab ts=8 sts=8: */
